@@ -65,7 +65,6 @@ type Editor struct {
 	monitorAndReadOnly         bool            // monitor the file for changes and open it as read-only
 	primaryClipboard           bool            // use the primary or the secondary clipboard on UNIX?
 	jumpToLetterMode           bool            // jump directly to a highlighted letter
-	nanoMode                   bool            // emulate GNU Nano
 	spellCheckMode             bool            // spell check mode?
 	createDirectoriesIfMissing bool            // when saving a file, should directories be created if they are missing?
 	displayQuickHelp           bool            // display the quick help box?
@@ -73,11 +72,13 @@ type Editor struct {
 	dirMode                    bool            // browse a directory and also interact with git
 	highlightCurrentLine       bool            // highlight the current line
 	highlightCurrentText       bool            // highlight the current text (not the entire line)
-	changed                    atomic.Bool     // has the contents changed, since last save?
-	redraw                     atomic.Bool     // if the contents should be redrawn in the next loop
-	redrawCursor               atomic.Bool     // if the cursor should be moved to the location it is supposed to be
-	drawProgress               atomic.Bool     // used for drawing the progress character on the right side
-	drawFuncName               atomic.Bool
+	// atomic.Bool are used for values that might be read when redrawing text asynchronously
+	changed      atomic.Bool // has the contents changed, since last save?
+	redraw       atomic.Bool // if the contents should be redrawn in the next loop
+	redrawCursor atomic.Bool // if the cursor should be moved to the location it is supposed to be
+	drawProgress atomic.Bool // used for drawing the progress character on the right side
+	drawFuncName atomic.Bool // used when drawing the function name in the top right corner
+	nanoMode     atomic.Bool // emulate GNU Nano
 }
 
 // Copy makes a copy of an Editor struct, with most fields deep copied
@@ -2285,7 +2286,7 @@ func (e *Editor) Switch(c *vt100.Canvas, tty *vt100.TTY, status *StatusBar, lk *
 		undo, switchUndoBackup = switchUndoBackup, undo
 	} else {
 		fnord := FilenameOrData{filenameToOpen, []byte{}, 0, false}
-		e2, statusMessage, displayedImage, err = NewEditor(tty, c, fnord, LineNumber(0), ColNumber(0), e.Theme, e.syntaxHighlight, false, e.monitorAndReadOnly, e.nanoMode, e.createDirectoriesIfMissing, e.displayQuickHelp)
+		e2, statusMessage, displayedImage, err = NewEditor(tty, c, fnord, LineNumber(0), ColNumber(0), e.Theme, e.syntaxHighlight, false, e.monitorAndReadOnly, e.nanoMode.Load(), e.createDirectoriesIfMissing, e.displayQuickHelp)
 		if err == nil { // no issue
 			// Save the current Editor to the switchBuffer if switchBuffer if empty, then use the new editor.
 			switchBuffer.Snapshot(e)
@@ -2410,10 +2411,29 @@ func (e *Editor) CurrentWord() string {
 	return string(word)
 }
 
+// AnyTextBeforeCursor checks if there is any text before the cursor, on the same line
+func (e *Editor) AnyTextBeforeCursor() bool {
+	y := int(e.DataY())
+	runes, ok := e.lines[y]
+	if !ok {
+		// This should never happen
+		return false
+	}
+
+	// Either find x or use the last index of the line
+	x, err := e.DataX()
+	if err != nil {
+		x = len(runes)
+	}
+
+	return strings.TrimSpace(string(runes[:x])) != ""
+}
+
 // LettersBeforeCursor returns the current word up until the cursor (for autocompletion)
 func (e *Editor) LettersBeforeCursor() string {
 	y := int(e.DataY())
 	runes, ok := e.lines[y]
+
 	if !ok {
 		// This should never happen
 		return ""
@@ -2530,7 +2550,7 @@ func (e *Editor) UserInput(c *vt100.Canvas, tty *vt100.TTY, status *StatusBar, t
 				status.SetMessage(title + ": " + entered)
 				status.ShowNoTimeout(c, e)
 			}
-		case "c:27", "c:3", "c:17", "c:24": // esc, ctrl-c, ctrl-q or ctrl-x
+		case "c:27", "c:3", "c:24": // esc, ctrl-c or ctrl-x
 			cancel = true
 			entered = ""
 			fallthrough // done
