@@ -17,7 +17,6 @@ import (
 	"github.com/xyproto/env/v2"
 	"github.com/xyproto/iferr"
 	"github.com/xyproto/mode"
-	"github.com/xyproto/syntax"
 	"github.com/xyproto/vt100"
 )
 
@@ -39,9 +38,7 @@ const (
 )
 
 // Create a LockKeeper for keeping track of which files are being edited
-var (
-	fileLock = NewLockKeeper(defaultLockFile)
-)
+var fileLock = NewLockKeeper(defaultLockFile)
 
 // Loop will set up and run the main loop of the editor
 // a *vt100.TTY struct
@@ -113,7 +110,13 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 		parentIsMan = &b
 	}
 	if *parentIsMan {
-		e.mode = mode.ManPage
+		// The parent process is "man", but NROFF_FILENAME has not been set:
+		// This means that ctrl-space has been pressed a second time when editing an Nroff file, so switch back to Nroff mode.
+		if env.No("NROFF_FILENAME") {
+			e.mode = mode.Nroff
+		} else {
+			e.mode = mode.ManPage
+		}
 	}
 
 	// Minor adjustments to some modes
@@ -126,7 +129,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 	}
 
 	// Prepare a status bar
-	status := NewStatusBar(e.StatusForeground, e.StatusBackground, e.StatusErrorForeground, e.StatusErrorBackground, e, statusDuration, messageAfterRedraw, e.nanoMode)
+	status := e.NewStatusBar(statusDuration, messageAfterRedraw)
 
 	e.SetTheme(e.Theme)
 
@@ -247,7 +250,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 		switch key {
 		case "c:17": // ctrl-q, quit
 
-			if e.nanoMode { // nano: ctrl-w, search backwards
+			if e.nanoMode.Load() { // nano: ctrl-w, search backwards
 				const clearPreviousSearch = true
 				const searchForward = false
 				e.SearchMode(c, status, tty, clearPreviousSearch, searchForward, undo)
@@ -257,7 +260,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			e.quit = true
 		case "c:23": // ctrl-w, format or insert template (or if in git mode, cycle interactive rebase keywords)
 
-			if e.nanoMode { // nano: ctrl-w, search
+			if e.nanoMode.Load() { // nano: ctrl-w, search
 				const clearPreviousSearch = true
 				const searchForward = true
 				e.SearchMode(c, status, tty, clearPreviousSearch, searchForward, undo)
@@ -339,7 +342,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 
 		case "c:6": // ctrl-f, search for a string
 
-			if e.nanoMode { // nano: ctrl-f, cursor forward
+			if e.nanoMode.Load() { // nano: ctrl-f, cursor forward
 				e.CursorForward(c, status)
 				break
 			}
@@ -369,7 +372,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			regularEditingRightNow = true
 
 		case "c:0": // ctrl-space, build source code to executable, or export, depending on the mode
-			if e.nanoMode {
+			if e.nanoMode.Load() {
 				break // do nothing
 			}
 
@@ -431,7 +434,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			// debug mode: next instruction
 
 			// Save the current file, but only if it has changed
-			if !e.nanoMode && e.changed.Load() {
+			if !e.nanoMode.Load() && e.changed.Load() {
 				if err := e.Save(c, tty); err != nil {
 					status.ClearAll(c, false)
 					status.SetError(err)
@@ -440,7 +443,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 				}
 			}
 
-			if e.nanoMode {
+			if e.nanoMode.Load() {
 				e.NanoNextTypo(c, status)
 				break
 			}
@@ -573,7 +576,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			e.redrawCursor.Store(true)
 		case "c:15": // ctrl-o, launch the command menu
 
-			if e.nanoMode { // ctrl-o, save
+			if e.nanoMode.Load() { // ctrl-o, save
 				// Ask the user which filename to save to
 				if newFilename, ok := e.UserInput(c, tty, status, "Save as", e.filename, []string{e.filename}, false, e.filename); ok {
 					e.filename = newFilename
@@ -591,13 +594,19 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			status.ClearAll(c, false)
 			undo.Snapshot(e)
 			undoBackup := undo
-			lastCommandMenuIndex = e.CommandMenu(c, tty, status, bookmark, undo, lastCommandMenuIndex, forceFlag, fileLock)
+			selectedIndex, spacePressed := e.CommandMenu(c, tty, status, bookmark, undo, lastCommandMenuIndex, forceFlag, fileLock)
+			lastCommandMenuIndex = selectedIndex
+			if spacePressed {
+				status.Clear(c, false)
+				// Ask the user for a command and run it
+				e.CommandPrompt(c, tty, status, bookmark, undo)
+			}
 			undo = undoBackup
 			regularEditingRightNow = true
 
 		case "c:31": // ctrl-_, jump to a matching parenthesis or enter a digraph
 
-			if e.nanoMode { // nano: ctrl-/
+			if e.nanoMode.Load() { // nano: ctrl-/
 				// go to line
 				e.JumpMode(c, status, tty)
 				e.redraw.Store(true)
@@ -669,7 +678,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 
 		case "c:16": // ctrl-p, scroll up or jump to the previous match, using the sticky search term. In debug mode, change the pane layout.
 
-			if !e.nanoMode {
+			if !e.nanoMode.Load() {
 				if e.debugMode {
 					// e.showRegisters has three states, 0 (SmallRegisterWindow), 1 (LargeRegisterWindow) and 2 (NoRegisterWindow)
 					e.debugShowRegisters++
@@ -781,7 +790,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 
 		case "c:14": // ctrl-n, scroll down or jump to next match, using the sticky search term
 
-			if !e.nanoMode {
+			if !e.nanoMode.Load() {
 
 				// If in Debug mode, let ctrl-n mean "next instruction"
 				if e.debugMode {
@@ -979,7 +988,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			e.redrawCursor.Store(true)
 
 		case "c:12": // ctrl-l, go to line number or percentage
-			if !e.nanoMode {
+			if !e.nanoMode.Load() {
 				regularEditingRightNow = false
 				e.ClearSearch() // clear the current search first
 				switch e.JumpMode(c, status, tty) {
@@ -1095,7 +1104,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			}
 			e.redraw.Store(true)
 
-		case "c:13": // return
+		case "c:13", "\n": // return
 
 			// Show a "Read only" status message if a man page is being viewed or if the editor is read-only
 			// It is an alternative way to quickly check if the file is read-only,
@@ -1117,9 +1126,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			undo.Snapshot(e)
 
 			var (
-				lineContents = e.CurrentLine()
-
-				trimmedLine              = strings.TrimSpace(lineContents)
+				trimmedLine              = e.TrimmedLine()
 				currentLeadingWhitespace = e.LeadingWhitespace()
 				// Grab the leading whitespace from the current line, and indent depending on the end of trimmedLine
 				leadingWhitespace = e.smartIndentation(currentLeadingWhitespace, trimmedLine, false) // the last parameter is "also dedent"
@@ -1379,59 +1386,57 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			y := int(e.DataY())
 			r := e.Rune()
 			leftRune := e.LeftRune()
-			ext := filepath.Ext(e.filename)
 
-			// Tab completion of words for Go
-			if word := e.LettersBeforeCursor(); e.mode != mode.Blank && e.mode != mode.GoAssembly && e.mode != mode.Assembly && leftRune != '.' && !unicode.IsLetter(r) && len(word) > 0 {
-				found := false
-				expandedWord := ""
-				for kw := range syntax.Keywords {
-					if len(kw) < 3 {
-						// skip too short suggestions
+			// Tab completion with Ollama
+			if ollamaClient != nil && e.mode != mode.Blank && e.AnyTextBeforeCursor() {
+
+				status.ClearAll(c, true)
+				status.SetMessage(fmt.Sprintf("Generating code with Ollama and the %s model...", codeCompletionModel))
+				status.ShowNoTimeout(c, e)
+				vt100.ShowCursor(false)
+
+				linesOfContext := ollamaContextLines / 2
+
+				currentLineIndex := int(e.LineIndex())
+
+				var codeBefore strings.Builder
+				for i := currentLineIndex - linesOfContext; i < currentLineIndex; i++ {
+					if i < 0 {
 						continue
 					}
-					if strings.HasPrefix(kw, word) {
-						if !found || (len(kw) < len(expandedWord)) && (len(expandedWord) > 0) {
-							expandedWord = kw
-							found = true
-						}
+					codeBefore.WriteString(e.Line(LineIndex(i)) + "\n")
+				}
+				codeBefore.WriteString(e.CurrentLine()) // without a newline
+
+				var codeAfter strings.Builder
+				for i := currentLineIndex + 1; i < currentLineIndex+linesOfContext; i++ {
+					if int(i) >= e.Len() {
+						break
 					}
+					codeAfter.WriteString(e.Line(LineIndex(i)) + "\n")
 				}
 
-				// Found a suitable keyword to expand to? Insert the rest of the string.
-				if found {
-					toInsert := strings.TrimPrefix(expandedWord, word)
-					undo.Snapshot(e)
-					e.redrawCursor.Store(true)
+				codeStart := codeBefore.String()
+				codeEnd := codeAfter.String()
+
+				if response, err := ollamaClient.GetBetweenResponse(codeStart, codeEnd); err == nil { // success
+
+					generatedCodeCompletion := strings.TrimSuffix(response.Response, "\n")
+					//logf("CODE START ---\n%s\n---\n\nGENERATED ---\n%s\n\n--- CODE END ---\n%s\n", codeStart, generatedCodeCompletion, codeEnd)
+					if generatedCodeCompletion != "" {
+						// Take an undo snapshot and insert the generated code
+						undo.Snapshot(e)
+						e.InsertStringAndMove(c, generatedCodeCompletion)
+					}
+
 					e.redraw.Store(true)
-					// Insert the part of expandedWord that comes after the current word
-					e.InsertStringAndMove(c, toInsert)
-					break
-				}
-
-				// Tab completion after a '.'
-			} else if word := e.LettersOrDotBeforeCursor(); e.mode != mode.Blank && e.mode != mode.GoAssembly && e.mode != mode.Assembly && leftRune == '.' && !unicode.IsLetter(r) && len(word) > 0 {
-				// Now the preceding word before the "." has been found
-
-				// Trim the trailing ".", if needed
-				word = strings.TrimSuffix(strings.TrimSpace(word), ".")
-
-				// Grep all files in this directory with the same extension as the currently edited file
-				// for what could follow the word and a "."
-				suggestions := corpus(word, "*"+ext)
-
-				// Choose a suggestion (tab cycles to the next suggestion)
-				chosen := e.SuggestMode(c, status, tty, suggestions)
-				e.redrawCursor.Store(true)
-				e.redraw.Store(true)
-
-				if chosen != "" {
-					undo.Snapshot(e)
-					// Insert the chosen word
-					e.InsertStringAndMove(c, chosen)
-					break
+					e.redrawCursor.Store(true)
+					e.RedrawAtEndOfKeyLoop(c, status, false, true)
+					e.redraw.Store(false)
+					e.redrawCursor.Store(false)
 				}
 
+				break
 			}
 
 			// Enable auto indent if the extension is not "" and either:
@@ -1548,7 +1553,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 
 		case "c:25": // ctrl-y
 
-			if e.nanoMode { // nano: ctrl-y, page up
+			if e.nanoMode.Load() { // nano: ctrl-y, page up
 				h := int(c.H())
 				e.redraw.Store(e.ScrollUp(c, status, h))
 				e.redrawCursor.Store(true)
@@ -1646,7 +1651,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			e.UserSave(c, tty, status)
 		case "c:7": // ctrl-g, either go to definition OR jump to matching parent/bracket OR toggle the status bar
 
-			if e.nanoMode { // nano: ctrl-g, help
+			if e.nanoMode.Load() { // nano: ctrl-g, help
 				status.ClearAll(c, false)
 				const repositionCursorAfterDrawing = true
 				e.DrawNanoHelp(c, repositionCursorAfterDrawing)
@@ -1701,7 +1706,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 
 		case "c:21": // ctrl-u to undo
 
-			if e.nanoMode { // nano: paste after cutting
+			if e.nanoMode.Load() { // nano: paste after cutting
 				e.Paste(c, status, &copyLines, &previousCopyLines, &firstPasteAction, &lastCopyY, &lastPasteY, &lastCutY, kh.PrevIs("c:13"))
 				break
 			}
@@ -1725,7 +1730,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			}
 		case "c:24": // ctrl-x, cut line
 
-			if e.nanoMode { // nano: ctrl-x, quit
+			if e.nanoMode.Load() { // nano: ctrl-x, quit
 
 				if e.changed.Load() {
 					// Ask the user which filename to save to
@@ -1787,7 +1792,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			e.End(c)
 		case "c:11": // ctrl-k, delete to end of line
 			undo.Snapshot(e)
-			if e.nanoMode { // nano: ctrl-k, cut line
+			if e.nanoMode.Load() { // nano: ctrl-k, cut line
 				// Prepare to cut
 				e.CutSingleLine(status, bookmark, &lastCutY, &lastCopyY, &lastPasteY, &copyLines, &firstCopyAction)
 				break
@@ -1795,7 +1800,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			e.DeleteToEndOfLine(c, status, bookmark, &lastCopyY, &lastPasteY, &lastCutY)
 		case "c:3", copyKey: // ctrl-c, copy the stripped contents of the current line
 
-			if e.nanoMode { // nano: ctrl-c, report cursor position
+			if e.nanoMode.Load() { // nano: ctrl-c, report cursor position
 				status.ClearAll(c, false)
 				status.NanoInfo(c, e)
 				break
@@ -1881,7 +1886,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 
 		case "c:22": // ctrl-v, paste
 
-			if e.nanoMode { // nano: ctrl-v, page down
+			if e.nanoMode.Load() { // nano: ctrl-v, page down
 				h := int(c.H())
 				e.redraw.Store(e.ScrollDown(c, status, h, h))
 				e.redrawCursor.Store(true)
@@ -1896,7 +1901,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 
 		case "c:18": // ctrl-r, to open or close a portal. In debug mode, continue running the program.
 
-			if e.nanoMode { // nano: ctrl-r, insert file
+			if e.nanoMode.Load() { // nano: ctrl-r, insert file
 				// Ask the user which filename to insert
 				if insertFilename, ok := e.UserInput(c, tty, status, "Insert file", "", []string{e.filename}, false, e.filename); ok {
 					err := e.RunCommand(c, tty, status, bookmark, undo, "insertfile", insertFilename)
@@ -1951,7 +1956,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			status.Show(c, e)
 		case "c:2": // ctrl-b, go back after jumping to a definition, bookmark, unbookmark or jump to bookmark. Toggle breakpoint if in debug mode.
 
-			if e.nanoMode { // nano: ctrl-b, cursor forward
+			if e.nanoMode.Load() { // nano: ctrl-b, cursor forward
 				e.CursorBackward(c, status)
 				break
 			}
@@ -2000,6 +2005,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 					status.SetMessage(s)
 				}
 			} else {
+				status.ClearAll(c, true)
 				if bookmark == nil {
 					// no bookmark, create a bookmark at the current line
 					bookmark = e.pos.Copy()
@@ -2033,7 +2039,7 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 				break
 			}
 			undo.Snapshot(e)
-			if e.nanoMode {
+			if e.nanoMode.Load() {
 				// Up to 999 times, join the current line with the next, until the next line is empty
 				joinCount := 0
 				for i := 0; i < 999; i++ {
@@ -2154,14 +2160,15 @@ func Loop(tty *vt100.TTY, fnord FilenameOrData, lineNumber LineNumber, colNumber
 			kh.Push(key)
 		}
 
-		// Display the ctrl-o menu if esc was pressed 4 times
-		if !e.nanoMode && kh.Repeated("c:27", 4-1) { // esc pressed 4 times (minus the one that was added just now)
+		// Display the ctrl-o menu if esc was pressed 4 times. Do not react if space is pressed.
+		if !e.nanoMode.Load() && kh.Repeated("c:27", 4-1) { // esc pressed 4 times (minus the one that was added just now)
 			backFunctions = make([]func(), 0)
 			regularEditingRightNow = false
 			status.ClearAll(c, false)
 			undo.Snapshot(e)
 			undoBackup := undo
-			lastCommandMenuIndex = e.CommandMenu(c, tty, status, bookmark, undo, lastCommandMenuIndex, forceFlag, fileLock)
+			selectedIndex, _ := e.CommandMenu(c, tty, status, bookmark, undo, lastCommandMenuIndex, forceFlag, fileLock)
+			lastCommandMenuIndex = selectedIndex
 			undo = undoBackup
 			// Reset the key history next iteration
 			clearKeyHistory = true
